@@ -41,6 +41,9 @@
 #ifdef WIN32
 #include <ws2tcpip.h>
 #endif
+#ifdef ENABLE_SHA512
+#include <openssl/sha.h>
+#endif
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -107,7 +110,7 @@ static const struct option_usage usage[] = {
 		"The special level 5 (or higher) includes xfer logging (lots of output)\n"
 	},
 	/** --libpcap */
-	{"interface",    0, OPT_STR,    {.str = "eth0"},
+	{"interface",    0, OPT_STR,    {.str = NULL},
 #ifndef HAVE_PCAP
 		"(Not available on this platform.)\n"
 #endif
@@ -131,6 +134,7 @@ static const struct option_usage usage[] = {
 	/** --passwd */
 	{"password",     0, OPT_STR,    {.str = NULL},
 		"Set a password (must be same on client and proxy)\n"
+		"DEPRECATED: Will be removed/replaced soon!\n"
 	},
 	/** --udp */
 	{NULL,           0, OPT_BOOL,   {.num = 0},
@@ -143,34 +147,21 @@ static const struct option_usage usage[] = {
 		"Unprivileged mode will only work on some systems, and is in general less reliable\n"
 		"than running in privileged mode.\n"
 	},
-	/** --window-size */
-	{"packets",      0, OPT_DEC32,  {.unum = 64},
-		"Tune the number of packets that can be in-flight at the same time.\n"
-		"Increasing the window size will improve the maximum potential bandwidth.\n"
-	},
-	/** --ack-interval */
-	{"milliseconds", 0, OPT_DEC32,  {.unum = 1000},
-		"Tune the explicit acknowledgement interval (in milliseconds)\n"
-		"Decreasing the acknowledgement interval can improve NAT stability.\n"
-	},
-	/** --resend-interval */
-	{"milliseconds", 0, OPT_DEC32,  {.unum = 1500},
-		"Tune the lost packet timeout (in milliseconds)\n"
-		"Decreasing the resend interval can compensate for frequent packet loss.\n"
-	},
-	/** --payload-size */
-	{"bytes",        0, OPT_DEC32,  {.unum = 1024},
-		"Tune the amount of data per packet (in bytes)\n"
-		"Decreasing the payload size can avoid corruption of large packets.\n"
-		"Increasing the payload size can compensate for out-of-order delivery.\n"
-	},
-	/** --empty-pings */
-	{"count",        0, OPT_DEC32,  {.unum = 0},
-		"Tune the number of empty pings to send with each explicit acknowledgement.\n"
-		"Empty pings can compensate for ICMP sequence number inspection.\n"
+	/** --force-sha512 */
+	{"force-sha512", 0, OPT_BOOL,   {.num = 0},
+		"Force SHA512 as challenge response checksum generator.\n"
+#ifdef ENABLE_SHA512
+		"This is the default for this configuration.\n"
+#else
+		"SHA512 is not available for this configuration.\n"
+#endif
 	},
 	/** --daemon */
+#ifdef __FreeBSD__
+	{"pidfile",      0, OPT_STR,    {.str = "/var/run/ptunnel.pid"},
+#else
 	{"pidfile",      0, OPT_STR,    {.str = "/run/ptunnel.pid"},
+#endif
 #ifdef WIN32
 		"(Not available on this platform.)\n"
 #endif
@@ -232,11 +223,7 @@ static struct option long_options[] = {
 	{"passwd",      required_argument, 0, 'P'},
 	{"udp",               no_argument, &opts.udp, 1},
 	{"unprivileged",      no_argument, &opts.unprivileged, 1},
-	{"window-size", required_argument, 0, 'w'},
-	{"ack-interval", required_argument, 0, 'a'},
-	{"resend-interval", required_argument, 0, 't'},
-	{"payload-size", required_argument, 0, 'y'},
-	{"empty-pings", required_argument, 0, 'E'},
+	{"force-sha512",      no_argument, &opts.force_sha512, 1},
 	{"daemon",      optional_argument, 0, 'd'},
 	{"syslog",            no_argument, 0, 'S'},
 	{"user",        optional_argument, 0, 'u'},
@@ -249,9 +236,14 @@ static struct option long_options[] = {
 
 
 static const void *get_default_optval(enum option_type opttype, const char *optname) {
+	(void) opttype;
+
 	for (unsigned i = 0; i < ARRAY_SIZE(long_options); ++i) {
-		if (strncmp(long_options[i].name, optname, BUFSIZ /* not optimal */) == 0) {
-			assert(usage[i].otype == opttype);
+		if (strncmp(long_options[i].name, optname, BUFSIZ /* not optimal */) == 0 &&
+		    strlen(long_options[i].name) == strlen(optname))
+		{
+			assert(usage[i].otype == opttype &&
+			       (usage[i].otype != OPT_STR || usage[i].str));
 			return &usage[i].str;
 		}
 	}
@@ -274,9 +266,6 @@ static void set_options_defaults(void) {
 	opts.given_dst_port  = *(uint32_t *)  get_default_optval(OPT_DEC32, "remote-port");
 	opts.max_tunnels     = *(uint32_t *)  get_default_optval(OPT_DEC32, "connections");
 	opts.log_level       = *(int *)       get_default_optval(OPT_DEC32, "verbosity");
-#ifdef HAVE_PCAP
-	opts.pcap_device     = strdup(*(char **)get_default_optval(OPT_STR, "libpcap"));
-#endif
 	opts.log_path        = strdup(*(char **)get_default_optval(OPT_STR, "logfile"));
 	opts.log_file        = stdout;
 	opts.print_stats     = *(int *)       get_default_optval(OPT_BOOL,  "statistics");
@@ -310,6 +299,7 @@ static void print_multiline(const char *prefix, const char *multiline) {
 	const char *start, *end;
 
 	start = multiline;
+	end = NULL;
 	do {
 		if (start) {
 			end = strstr(start, sep);
@@ -421,7 +411,7 @@ int parse_options(int argc, char **argv) {
          *        since you have to pass long options as '--option=value'. Commonly used
          *        '--option value' is *NOT* allowed for some libc implementations.
          */
-		c = getopt_long(argc, argv, "m:p:l:r::R::c:v:L::o::sP:d::Su::g::C::e::w:a:t:y:E:h", &long_options[0], &oidx);
+		c = getopt_long(argc, argv, "m:p:l:r::R::c:v:L:o::sP:d::Su::g::C::e::h", &long_options[0], &oidx);
 		if (c == -1) break;
 
 		switch (c) {
@@ -498,11 +488,15 @@ int parse_options(int argc, char **argv) {
 				if (opts.password)
 					free(opts.password);
 				opts.password = strdup(optarg);
-				pt_log(kLog_debug, "Password set - unauthenicated connections will be refused.\n");
-				//  Compute the password digest
+				pt_log(kLog_debug, "%s\n", "Password set - unauthenicated connections will be refused.");
+				/* Compute the md5 password digest */
 				md5_init(&state);
-				md5_append(&state, (md5_byte_t*)optarg, strnlen(opts.password, BUFSIZ /* not optimal */));
-				md5_finish(&state, &opts.password_digest[0]);
+				md5_append(&state, (md5_byte_t *)optarg, strnlen(opts.password, BUFSIZ /* not optimal */));
+				md5_finish(&state, &opts.md5_password_digest[0]);
+#ifdef ENABLE_SHA512
+				pt_log(kLog_debug, "%s\n", "Password set - sha512 authentication enabled.");
+				SHA512((const unsigned char *)optarg, strnlen(opts.password, BUFSIZ /* not optimal */), &opts.sha512_password_digest[0]);
+#endif
 				//  Hide the password in process listing
 				memset(optarg, '*', strnlen(optarg, BUFSIZ /* not optimal */));
 				break;
@@ -570,31 +564,6 @@ int parse_options(int argc, char **argv) {
 				pt_log(kLog_error, "SeLinux: %s\n", "feature not supported");
 				exit(1);
 #endif
-			case 'w':
-				if (!optarg)
-					break;
-				opts.window_size = atoi(optarg);
-				break;
-			case 'a':
-				if (!optarg)
-					break;
-				opts.ack_interval = atoi(optarg);
-				break;
-			case 't':
-				if (!optarg)
-					break;
-				opts.resend_interval = atoi(optarg);
-				break;
-			case 'y':
-				if (!optarg)
-					break;
-				opts.payload_size = atoi(optarg);
-				break;
-			case 'E':
-				if (!optarg)
-					break;
-				opts.empty_pings = atoi(optarg);
-				break;
 			case 'h':
 				print_usage(argv[0]);
 				exit(EXIT_SUCCESS);
@@ -609,6 +578,13 @@ int parse_options(int argc, char **argv) {
 		exit(1);
 	}
 
+#ifndef ENABLE_SHA512
+	if (opts.force_sha512) {
+		pt_log(kLog_error, "%s\n", "You are forcing sha512 but it isn't available.");
+		return 1;
+	}
+#endif
+
 	if (opts.given_proxy_hostname) {
 		if ((ret = host_to_addr(opts.given_proxy_hostname, &opts.given_proxy_ip)) != 0) {
 			pt_log(kLog_error, "Failed to look up %s as destination address: %s\n",
