@@ -53,7 +53,12 @@
 #include <selinux/selinux.h>
 #endif
 
+#ifndef PACKAGE_STRING
+#define PACKAGE_STRING "ptunnel-ng"
+#endif
+
 #ifdef WIN32
+#include <tchar.h>
 #include <winsock2.h>
 /* Map errno (which Winsock doesn't use) to GetLastError; include the code in the strerror */
 #ifdef errno
@@ -75,6 +80,25 @@ static char * print_last_windows_error() {
 #define strerror(x) print_last_windows_error()
 #endif /* WIN32 */
 
+#ifdef HAVE_NPCAP
+static BOOL LoadNpcapDlls()
+{
+	TCHAR npcap_dir[512];
+	UINT len;
+	len = GetSystemDirectory(npcap_dir, 480);
+	if (!len) {
+		pt_log(kLog_error, "Error in GetSystemDirectory: %x", GetLastError());
+		return FALSE;
+	}
+	_tcscat_s(npcap_dir, 512, _T("\\Npcap"));
+	if (SetDllDirectory(npcap_dir) == 0) {
+		pt_log(kLog_error, "Error in SetDllDirectory: %x", GetLastError());
+		return FALSE;
+	}
+	return TRUE;
+}
+#endif
+
 /* globals */
 /** Lock protecting the chain of connections */
 pthread_mutex_t chain_lock;
@@ -85,9 +109,10 @@ int num_threads = 0;
 /** Current tunnel count */
 uint32_t num_tunnels = 0;
 /** Table indicating when a connection ID is allowable (used by proxy) */
-uint32_t *seq_expiry_tbl = NULL;
+time_t *seq_expiry_tbl = NULL;
 
 /* Some buffer constants */
+const int tcp_receive_buf_len  = kDefault_buf_size;
 const int icmp_receive_buf_len = kDefault_buf_size + kIP_header_size +
                                  kICMP_header_size + sizeof(ping_tunnel_pkt_t);
 const int pcap_buf_size        = (kDefault_buf_size + kIP_header_size +
@@ -150,12 +175,18 @@ int main(int argc, char *argv[]) {
 	}
 #endif /* WIN32 */
 
-	memset(opts.password_digest, 0, kMD5_digest_size);
+#ifdef HAVE_NPCAP
+	if (!LoadNpcapDlls())
+		return -1;
+#endif
+
+	memset(opts.md5_password_digest, 0, kMD5_digest_size);
+	memset(opts.sha512_password_digest, 0, kSHA512_digest_size);
 
 	/* The seq_expiry_tbl is used to prevent the remote ends from prematurely
 	 * re-using a sequence number.
 	 */
-	seq_expiry_tbl = (uint32_t *) calloc(65536, sizeof(uint32_t));
+	seq_expiry_tbl = (time_t *) calloc(65536, sizeof(time_t));
 
 	/* Parse options */
 	if (parse_options(argc, argv))
@@ -181,10 +212,10 @@ int main(int argc, char *argv[]) {
 	}
 #ifdef WIN32
 	if (!opts.pcap && !opts.udp) {
-		pt_log(kLog_info, "Running ptunnel-ng on Windows in ICMP mode without WinPcap enabled is not supported and may not work!\n");
-        pt_log(kLog_info, "If you encounter problems, install WinPCAP from:\n");
-		pt_log(kLog_info, "https://www.winpcap.org/install/default.htm or for WIN10: https://nmap.org/npcap/windows-10.html\n");
-		pt_log(kLog_info, "After WinPCAP is installed, you can list pcap devices with: --list-pcap-devices\n");
+		pt_log(kLog_info, "Running ptunnel-ng on Windows in ICMP mode without WinPcap/Npcap enabled is not supported and may not work!\n");
+        pt_log(kLog_info, "If you encounter problems, install WinPCAP/Npcap from:\n");
+		pt_log(kLog_info, "https://www.winpcap.org/install/default.htm or Npcap for WIN10: https://nmap.org/npcap/windows-10.html\n");
+		pt_log(kLog_info, "After WinPCAP is installed, you can list pcap devices with: --list-libpcap-devices\n");
 	}
 #endif
 #endif
@@ -252,8 +283,8 @@ int main(int argc, char *argv[]) {
 	}
 #endif /* !WIN32 */
 
-  	pthread_mutex_init(&chain_lock, 0);
-  	pthread_mutex_init(&num_threads_lock, 0);
+	pthread_mutex_init(&chain_lock, 0);
+	pthread_mutex_init(&num_threads_lock, 0);
 
 	//	Check mode, validate arguments and start either client or proxy.
 	if (opts.mode == kMode_forward) {
@@ -349,7 +380,7 @@ void pt_forwarder(void) {
 #ifndef WIN32
 				if (pthread_create(&pid, 0, pt_proxy, 0) != 0)
 #else
-				if (0 == (pid = _beginthreadex(0, 0, (unsigned int (__stdcall *)(void *))pt_proxy, 0, 0, 0)))
+				if (0 == (pid = _beginthreadex(0, 0, pt_proxy, 0, 0, 0)))
 #endif
 				{
 					pt_log(kLog_error, "Couldn't create thread! Dropping incoming connection.\n");
@@ -359,7 +390,7 @@ void pt_forwarder(void) {
 				}
 			}
 			addr	= dest_addr;
-			rand_id	= (uint16_t) pt_random();
+			rand_id	= pt_random();
 			create_and_insert_proxy_desc(rand_id, rand_id, new_sock, &addr, opts.given_dst_ip, opts.given_dst_port, kProxy_start, kUser_flag);
 			pthread_mutex_unlock(&num_threads_lock);
 		}
@@ -401,7 +432,14 @@ int pt_create_udp_socket(int port) {
 
 /* pt_proxy: This function does all the client and proxy stuff.
  */
-void* pt_proxy(void *args) {
+#ifndef WIN32
+void * pt_proxy(void *args)
+#else
+unsigned int __stdcall pt_proxy(void *args)
+#endif
+{
+	(void) args;
+
 	fd_set             set;
 	struct timeval     timeout;
 	int                bytes;
@@ -598,29 +636,7 @@ void* pt_proxy(void *args) {
 			if (cur->state == kProxy_start) {
 				pt_log(kLog_verbose, "Sending proxy request.\n");
 				cur->last_ack = time_as_double();
-				uint16_t extended_options[4];
-				size_t extended_options_size = 0;
-				memset(extended_options, 0, sizeof(extended_options));
-				if (opts.window_size) {
-					extended_options_size = sizeof(uint16_t);
-					extended_options[0] = htons(opts.window_size);
-				}
-				if (opts.ack_interval) {
-					extended_options_size = 2*sizeof(uint16_t);
-					extended_options[1] = htons(opts.ack_interval);
-				}
-				if (opts.resend_interval) {
-					extended_options_size = 3*sizeof(uint16_t);
-					extended_options[2] = htons(opts.resend_interval);
-				}
-				if (opts.payload_size) {
-					extended_options_size = 4*sizeof(uint16_t);
-					extended_options[3] = htons(opts.payload_size);
-				}
-				queue_packet(fwd_sock, cur->pkt_type, (char *)extended_options, extended_options_size, cur->id_no, cur->id_no,
-				             &cur->my_seq, cur->send_ring, &cur->send_idx, &cur->send_wait_ack,
-				             cur->dst_ip, cur->dst_port, cur->state | cur->type_flag,
-				             &cur->dest_addr, cur->next_remote_seq, &cur->send_first_ack, &cur->ping_seq, cur->window_size);
+				queue_packet(fwd_sock, cur, NULL, 0, cur->dst_ip, cur->dst_port, cur->state | cur->type_flag);
 				cur->xfer.icmp_out++;
 				cur->state = kProto_data;
 			}
@@ -634,7 +650,7 @@ void* pt_proxy(void *args) {
 			}
 			/* Handle TCP traffic */
 			if (FD_ISSET(cur->sock, &set)) {
-				bytes = recv(cur->sock, cur->buf, cur->payload_size, 0);
+				bytes = recv(cur->sock, cur->buf, tcp_receive_buf_len, 0);
 				if (bytes <= 0) {
 					pt_log(kLog_info, "Connection closed or lost.\n");
 					tmp	= cur->next;
@@ -647,10 +663,7 @@ void* pt_proxy(void *args) {
 				}
 				cur->xfer.bytes_out	+= bytes;
 				cur->xfer.icmp_out++;
-				queue_packet(fwd_sock, cur->pkt_type, cur->buf, bytes, cur->id_no,
-				             cur->icmp_id, &cur->my_seq, cur->send_ring, &cur->send_idx,
-				             &cur->send_wait_ack, 0, 0, cur->state | cur->type_flag,
-				             &cur->dest_addr, cur->next_remote_seq, &cur->send_first_ack, &cur->ping_seq, cur->window_size);
+				queue_packet(fwd_sock, cur, cur->buf, bytes, 0, 0, cur->state | cur->type_flag);
 			}
 			prev = cur;
 			tmp  = cur->next;
@@ -676,7 +689,7 @@ void* pt_proxy(void *args) {
 		for (cur = chain; cur; cur = cur->next) {
 			in_addr.s_addr = cur->dst_ip;
 			if (cur->last_activity + kAutomatic_close_timeout < now) {
-				pt_log(kLog_info, "Dropping tunnel to %s:%d due to inactivity.\n", inet_ntoa(in_addr), cur->dst_port, cur->id_no);
+				pt_log(kLog_info, "Dropping tunnel %u to %s:%u due to inactivity.\n", cur->id_no, inet_ntoa(in_addr), cur->dst_port);
 				cur->should_remove = 1;
 				continue;
 			}
@@ -702,26 +715,9 @@ void* pt_proxy(void *args) {
 			if (cur->last_ack+cur->ack_interval < now && cur->send_wait_ack < cur->window_size &&
 			    cur->remote_ack_val+1 != cur->next_remote_seq)
 			{
-				idx = cur->send_idx;
-				queue_packet(fwd_sock, cur->pkt_type, 0, 0, cur->id_no, cur->icmp_id,
-				             &cur->my_seq, cur->send_ring, &cur->send_idx, &cur->send_wait_ack,
-				             cur->dst_ip, cur->dst_port, kProto_ack | cur->type_flag,
-				             &cur->dest_addr, cur->next_remote_seq, &cur->send_first_ack, &cur->ping_seq, cur->window_size);
-				cur->xfer.icmp_ack_out++;
-				if (opts.empty_pings &&
-					cur->last_data_activity > cur->last_ack &&
-					cur->send_ring[idx].pkt_len > sizeof(icmp_echo_packet_t) &&
-					cur->send_ring[idx].pkt->type == kICMP_echo_request) {
-					for (uint16_t e = 0; e < opts.empty_pings; e++) {
-						cur->send_ring[idx].pkt->seq      = htons(cur->ping_seq);
-						cur->ping_seq++;
-						cur->send_ring[idx].pkt->checksum = 0;
-						cur->send_ring[idx].pkt->checksum = htons(calc_icmp_checksum((uint16_t*)cur->send_ring[idx].pkt, sizeof(icmp_echo_packet_t)));
-						sendto(fwd_sock, (const void*)cur->send_ring[idx].pkt, sizeof(icmp_echo_packet_t),
-						       0, (struct sockaddr*)&cur->dest_addr, sizeof(struct sockaddr));
-					}
-				}
+				queue_packet(fwd_sock, cur, NULL, 0, cur->dst_ip, cur->dst_port, kProto_ack | cur->type_flag);
 				cur->last_ack = now;
+				cur->xfer.icmp_ack_out++;
 			}
 		}
 		pthread_mutex_unlock(&chain_lock);
@@ -864,14 +860,12 @@ uint16_t calc_icmp_checksum(uint16_t *data, int bytes) {
  * the tunnel is being closed down.
  */
 void send_termination_msg(proxy_desc_t *cur, int icmp_sock) {
+	size_t i;
+	const size_t max_termination_msgs = 3;
+
 	/* Send packet twice, hoping at least one of them makes it through.. */
-	queue_packet(icmp_sock, cur->pkt_type, 0, 0, cur->id_no, cur->icmp_id, &cur->my_seq,
-	             cur->send_ring, &cur->send_idx, &cur->send_wait_ack, 0, 0,
-	             kProto_close | cur->type_flag, &cur->dest_addr, cur->next_remote_seq,
-	             &cur->send_first_ack, &cur->ping_seq, cur->window_size);
-	queue_packet(icmp_sock, cur->pkt_type, 0, 0, cur->id_no, cur->icmp_id, &cur->my_seq,
-	             cur->send_ring, &cur->send_idx, &cur->send_wait_ack, 0, 0,
-	             kProto_close | cur->type_flag, &cur->dest_addr, cur->next_remote_seq,
-	             &cur->send_first_ack, &cur->ping_seq, cur->window_size);
-	cur->xfer.icmp_out += 2;
+	for (i = 0; i < max_termination_msgs; ++i) {
+		queue_packet(icmp_sock, cur, NULL, 0, cur->dst_ip, cur->dst_port, kProto_close | cur->type_flag);
+	}
+	cur->xfer.icmp_out += max_termination_msgs;
 }
